
#include "Config.h"
#include "MotionControl.h"
#include "Serial.h"

#include <Arduino.h>
#include <ESP32Servo.h>

Servo z_servo;

// === Initialization ===
void motion_init() {
  pinMode(X_STEP_PIN, OUTPUT);
  pinMode(X_DIR_PIN, OUTPUT);
  pinMode(Y_STEP_PIN, OUTPUT);
  pinMode(Y_DIR_PIN, OUTPUT);
}

void z_move_to(float z_target) {
  // Mapear de 0-10 → 0-360 grados
  float clamped = constrain(z_target, 0.0, 10.0);  // Limita entre 0 y 10
  int angle = map(clamped * 100, 0, 1000, 0, 360); // Multiplico por 100 para map() con enteros

  Serial.printf("↕ Moving Z (servo) to angle: %d°\n", angle);
  z_servo.write(angle);
  delay(500);  // Dar tiempo a que se complete el giro
}

const float steps_per_mm[N_AXIS] = {
  DEFAULT_X_STEPS_PER_MM,
  DEFAULT_Y_STEPS_PER_MM,
  DEFAULT_Z_STEPS_PER_MM
};

const int dir_pin[2] = { X_DIR_PIN, Y_DIR_PIN };
const int step_pin[2] = { X_STEP_PIN, Y_STEP_PIN };

const float max_travel[N_AXIS] = {
  DEFAULT_X_MAX_TRAVEL,
  DEFAULT_Y_MAX_TRAVEL,
  DEFAULT_Z_MAX_TRAVEL
};

float current_position[N_AXIS] = {0.0, 0.0, 0.0};
bool absolute_mode = true;

unsigned long step_delay_us = 1000;

void motion_execute_move(float x, float y, float z, float f) {
  Serial.println("== Begin motion ==");

  float target[N_AXIS];
  target[X_AXIS] = isnan(x) ? current_position[X_AXIS] : (absolute_mode ? x : current_position[X_AXIS] + x);
  target[Y_AXIS] = isnan(y) ? current_position[Y_AXIS] : (absolute_mode ? y : current_position[Y_AXIS] + y);
  target[Z_AXIS] = isnan(z) ? current_position[Z_AXIS] : (absolute_mode ? z : current_position[Z_AXIS] + z);

  Serial.printf("Target: X=%.2f Y=%.2f Z=%.2f\n", target[0], target[1], target[2]);

  if (!motion_check_limits(target)) {
    Serial.println("Error: Limit exceeded");
    return;
  }

  if (!isnan(z)) {
    Serial.println("↕ Moving Z (servo)");
    z_move_to(target[Z_AXIS]);
  }

  float delta_x = target[X_AXIS] - current_position[X_AXIS];
  float delta_y = target[Y_AXIS] - current_position[Y_AXIS];
  long steps_x = abs(MM_TO_STEPS_X(delta_x));
  long steps_y = abs(MM_TO_STEPS_Y(delta_y));

  int dir_x = (delta_x >= 0) ? HIGH : LOW;
  int dir_y = (delta_y >= 0) ? HIGH : LOW;
  digitalWrite(dir_pin[X_AXIS], dir_x);
  digitalWrite(dir_pin[Y_AXIS], dir_y);

  float distance_mm = sqrt(delta_x * delta_x + delta_y * delta_y);

  if (f > 0 && distance_mm > 0) {
    float mm_per_step = distance_mm / max(steps_x, steps_y);
    step_delay_us = (60000000.0 / f) * mm_per_step;
    if (step_delay_us < 200) step_delay_us = 200;
  } else {
    step_delay_us = 5000;
  }

  long steps = max(steps_x, steps_y);
  long counter_x = 0;
  long counter_y = 0;

  unsigned long last_step_time = micros();
  for (long i = 0; i < steps; i++) {
    while (micros() - last_step_time < step_delay_us);
    last_step_time = micros();

    counter_x += steps_x;
    counter_y += steps_y;

    if (counter_x >= steps) {
      digitalWrite(step_pin[X_AXIS], HIGH);
      delayMicroseconds(10);
      digitalWrite(step_pin[X_AXIS], LOW);
      counter_x -= steps;
    }

    if (counter_y >= steps) {
      digitalWrite(step_pin[Y_AXIS], HIGH);
      delayMicroseconds(10);
      digitalWrite(step_pin[Y_AXIS], LOW);
      counter_y -= steps;
    }
  }

  current_position[X_AXIS] = target[X_AXIS];
  current_position[Y_AXIS] = target[Y_AXIS];
  current_position[Z_AXIS] = target[Z_AXIS];

  Serial.println("Motion complete.");
}

void motion_go_home() {
  motion_execute_move(0.0, 0.0, 0.0, 0.0);
}

void motion_set_absolute_mode(bool mode) {
  absolute_mode = mode;
}

bool motion_check_limits(float* target) {
  for (uint8_t axis = 0; axis < N_AXIS; axis++) {
    if (target[axis] < 0 || target[axis] > max_travel[axis]) {
      return false;
    }
  }
  return true;
}
